#ifndef LIBWIISPRITE_IMAGE
#define LIBWIISPRITE_IMAGE

#include <stdlib.h>
#include <gccore.h>

//!libwiisprite namespace
namespace wsp{
	
	//!Error codes when loading pngs.
	enum IMG_LOAD_ERROR{
		IMG_LOAD_ERROR_NONE = 0, //!< Image successfully loaded.
		IMG_LOAD_ERROR_NOT_FOUND, //!< File was not found.
		IMG_LOAD_ERROR_INV_PNG, //!< Invalid PNG header.
		IMG_LOAD_ERROR_PNG_FAIL, //!< Invalid PNG data.
		IMG_LOAD_ERROR_WRONG_SIZE //!< The images width and height should both be a multiple of 4.
	};

	//!Stores imagedata and is capable of loading pngs.
	class Image{
		public:
			//!Constructor.
			Image();
			//!Destructor.
			~Image();

			//!Loads an image from a file. Width and height have to be a multiple of 4, or it won't load.
			//!Uses fopen(), so some basic filesystem initialization is required.
			//!\param path The path to the file.
			//!\return An error code based on loading status.
			IMG_LOAD_ERROR LoadImage(const char* path);
			
			//!Gets the width of this image.
			//!\return The width of the image. Returns 0 if no image is loaded.
			u32 GetWidth() const;
			//!Gets the height of this image.
			//!\return The height of the image. Returns 0 if no image is loaded.
			u32 GetHeight() const;
	
			//!Checks if the image is loaded.
			//!\return true if an image is stored, false if not.
			bool IsInitialized() const;

			//!Used to select this image for the next texturing process.
			//!Most of the time you won't gonna use this.
			void BindTexture();
		protected:
		private:
			u32 _width, _height;
			bool _initialized;
			GXTexObj _texObj; u8* _pixels;
	};
};

#endif
