#ifndef LIBWIISPRITE_LAYERMANAGER
#define LIBWIISPRITE_LAYERMANAGER

#include <stdlib.h>
#include <gccore.h>
#include "layer.h"

//!libwiisprite namespace
namespace wsp{
	//!Groups layers into one block in which they can be managed.
	class LayerManager{
		public:
			//!Constructor.
			//!\param boundary Specifies how many layers can be grouped into this manager.
			LayerManager(u32 boundary);
			//!Destructor.
			virtual ~LayerManager();

			//!Appends a layer at the end, thus drawing it at last.
			//!\param layer The layer to append. If it is already in the list, it gets removed first.
			void Append(Layer* layer);
			//!Inserts a layer into the manager. 
			//!\param layer The layer to insert. If it is already in the list, it gets removed first.
			//!\param index The new index of the layer. Can't be bigger than GetSize().
			void Insert(Layer* layer, u32 index);
			//!Removes a layer from the list.
			//!\param layer A layer that is in the list.
			void Remove(Layer* layer);

			//!Returns a layer at a specified index.
			//!\param index The index from where to poll the layer. Can't be bigger than GetSize().
			//!\return A pointer to the layer at the index. NULL if index is out of bounds.
			Layer* GetLayerAt(u32 index) const;
			//!Returns the size of the list of layers.
			//!\return The size of the current layerlist.
			u32 GetSize() const;

			//!Specifies where to view the current scene.
			//!\param x The upper left corners X position.
			//!\param y The upper left corners Y position.
			//!\param width The width of the window.
			//!\param height The height of the window.
			void SetViewWindow(s32 x, s32 y, u32 width, u32 height);
			
			//!Draws all the layers in this LayerManager.
			//!\param x The X offset for drawing.
			//!\param y The Y offset for drawing.
			void Draw(s32 x, s32 y) const;
		protected:
			u32 _width, //!< The width of the viewwindow.
				_height; //!< The height of the viewwindow.
			s32 _x, //!< The X position of the viewwindow.
				_y; //!< The Y position of the viewwindow.
		private:
			Layer** _layers;
			u32 _size,
				_boundary;
	};
};

#endif
