#include <stdio.h>
#include <stdlib.h>
#include <gccore.h>
#include <wiiuse/wpad.h>
#include <fat.h>

#include <wiisprite.h> // The main libwiisprite header.
using namespace wsp; // To not make us type this again and again

GameWindow gwd; // Initializes and renders our scene.
Sprite sprite; // The drawable object we can modify.
Sprite sprite2; // Another drawable object we can modify.
Image image; // Holds our image/texture.

// Entry point
int main(int argc, char **argv) {
	// Initialize filesystem to load from SD Card
	fatInitDefault();

	// Initialize Video
	gwd.InitVideo();
	
	// We initialize our layermanager with max. 2 sprites, just for our testing purposes.
	LayerManager manager(2); // DO NOT FORGET TO INCREASE THIS NUMBER FOR YOUR OWN TESTS.

	// Now we're loading an image.
	// If the image is not found at our location, we exit to the loader
	if(image.LoadImage("libwiisprite.png") != IMG_LOAD_ERROR_NONE)exit(0);
	// We assign the same loaded image to the sprites
	sprite.SetImage(&image);
	sprite2.SetImage(&image);
	// We're gonna move the first sprite with the pointer
	sprite.SetPosition(0, 0);
	// Fix the second sprite to a position at the center of the on the screen.
	sprite2.SetPosition(320-sprite2.GetWidth()/2, 240-sprite2.GetHeight()/2);
	// And load both sprites into the LayerManager
	manager.Append(&sprite); // Gets drawn the closest.
	manager.Append(&sprite2); // Gets drawn the farthest.


	// This function initialises the attached controllers
	WPAD_Init();
	// And this one gives us access to the accelerometers and infrared.
	WPAD_SetDataFormat(WPAD_CHAN_0, WPAD_FMT_BTNS_ACC_IR);
	while(1) {
		// Call WPAD_ScanPads each loop, this reads the latest controller states
		WPAD_ScanPads();

		// WPAD_ButtonsDown tells us which buttons were pressed in this loop
		// this is a "one shot" state which will not fire again until the button has been released
		u32 pressed = WPAD_ButtonsHeld(WPAD_CHAN_0);

		// We return to the launcher application via exit
		if(pressed & WPAD_BUTTON_HOME)exit(0);

		// With plus and minus we can zoom the sprite in or out
		if(pressed & WPAD_BUTTON_MINUS)sprite.SetZoom(sprite.GetZoom()-0.1f);
		if(pressed & WPAD_BUTTON_PLUS)sprite.SetZoom(sprite.GetZoom()+0.1f);

		// With A and B we can modify our transparency, just for fun.
		// We have to do the checks, so our program doesn't crash.
		if(pressed & WPAD_BUTTON_A && sprite.GetTransparency() < 0xff-4)sprite.SetTransparency(sprite.GetTransparency()+5);
		if(pressed & WPAD_BUTTON_B && sprite.GetTransparency() > 4)sprite.SetTransparency(sprite.GetTransparency()-5);

		// Infrared calculation
		ir_t ir; // The struct for infrared
		WPAD_IR(WPAD_CHAN_0, &ir); // Let's get our infrared data
		// Give our sprite the positions and the angle.
		sprite.SetPosition(ir.sx-200, ir.sy-250); // We use these constants to translate the position correctly to the screen
		sprite.Move(-((f32)sprite.GetWidth()/2), -((f32)sprite.GetHeight()/2)); // And these to make our image appear at the center of this position.
		sprite.SetRotation(ir.angle/2); // Set angle/2 to translate correctly

		// Draw everything what's in the manager.
		manager.Draw(0, 0); 

		// Finish rendering of everything.
		gwd.Flush();
	}

	return 0;
}
